import java.io.Closeable;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.change_vision.jude.api.inf.exception.LicenseNotFoundException;
import com.change_vision.jude.api.inf.exception.NonCompatibleException;
import com.change_vision.jude.api.inf.exception.ProjectLockedException;
import com.change_vision.jude.api.inf.exception.ProjectNotFoundException;
import com.change_vision.jude.api.inf.model.IClass;
import com.change_vision.jude.api.inf.model.IDiagram;
import com.change_vision.jude.api.inf.model.INamedElement;
import com.change_vision.jude.api.inf.model.IOperation;
import com.change_vision.jude.api.inf.model.IPackage;
import com.change_vision.jude.api.inf.model.IStateMachineDiagram;
import com.change_vision.jude.api.inf.project.ProjectAccessor;
import com.change_vision.jude.api.inf.project.ProjectAccessorFactory;


public class AstahDiagramFinder implements Closeable {

	ProjectAccessor prjAccessor;
	
	public AstahDiagramFinder(String inputFile) throws LicenseNotFoundException, ProjectNotFoundException, NonCompatibleException, IOException, ProjectLockedException, Throwable {
		prjAccessor = ProjectAccessorFactory.getProjectAccessor();
		prjAccessor.open(inputFile, true, false, true);
	}
	
	public void close() {
	    prjAccessor.close();
	}

	public List<IStateMachineDiagram> findAllStateMachineDiagrams() throws ProjectNotFoundException {
		return getAllStateMachineDiagrams(prjAccessor.getProject());
	}

    private List<IStateMachineDiagram> getAllStateMachineDiagrams(INamedElement iNamedElement) {
    	ArrayList<IStateMachineDiagram> diagrams = new ArrayList<IStateMachineDiagram>();
    	IDiagram[] dgms = iNamedElement.getDiagrams();
    	for (int i = 0; i < dgms.length; i++) {
			if (dgms[i] instanceof IStateMachineDiagram) {
				diagrams.add((IStateMachineDiagram) dgms[i]);
			}
    	}
    	if (iNamedElement instanceof IPackage) {
        	INamedElement[] children = ((IPackage)iNamedElement).getOwnedElements();
        	for (int i = 0; i < children.length; i++) {
        		diagrams.addAll(getAllStateMachineDiagrams(children[i]));
        	}
    	}
    	if (iNamedElement instanceof IClass) {
    		IOperation[] children = ((IClass)iNamedElement).getOperations();
        	for (int i = 0; i < children.length; i++) {
        		diagrams.addAll(getAllStateMachineDiagrams(children[i]));
        	}
    	}
    	return diagrams;
    }
}
