import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;

import com.change_vision.jude.api.inf.exception.LicenseNotFoundException;
import com.change_vision.jude.api.inf.exception.NonCompatibleException;
import com.change_vision.jude.api.inf.exception.ProjectLockedException;
import com.change_vision.jude.api.inf.exception.ProjectNotFoundException;
import com.change_vision.jude.api.inf.model.IStateMachineDiagram;

import freemarker.template.Configuration;
import freemarker.template.DefaultObjectWrapper;
import freemarker.template.Template;
import freemarker.template.TemplateException;

public class CourseCodeGenerator {

	/**
	 * @param args
	 *            [0]F*.astãpX
	 * @throws Throwable
	 * @throws ProjectLockedException
	 * @throws NonCompatibleException
	 * @throws ProjectNotFoundException
	 * @throws LicenseNotFoundException
	 */
	public static void main(String[] args) throws LicenseNotFoundException,
			ProjectNotFoundException, NonCompatibleException,
			ProjectLockedException, Throwable {

		// R}hCc[IvV擾
		Options options = new Options();
		options.addOption("o", true, "display current time");

		CommandLineParser parser = new PosixParser();
		CommandLine cmd = parser.parse(options, args);

		if (cmd.getArgs().length < 1) {
			System.err.println("Usage: CourseCodeGenerator.bat -o outputdir astahfile");
			return;
		}

		final String outputDir = cmd.hasOption("o") ? cmd.getOptionValue("o") : ".";
		final String inputFile = cmd.getArgs()[0];

		// ev[gGWRtBO
		Configuration templateConfig = createTemplateConfig();

		// \[XR[h𐶐		
		AstahDiagramFinder finder = null;
		try {
			// Xe[g}V}
			finder = new AstahDiagramFinder(inputFile);
			List<IStateMachineDiagram> diagrams = finder.findAllStateMachineDiagrams();

			// Xe[g}V}ƂɃt@C
			for (IStateMachineDiagram diagram : diagrams) {

				// ev[ǧɂȂ}bv𐶐
				Map<String, Object> templateRootMap = createTemplateRootMap(diagram);

				// \[Xt@C̐
				String sourceFilename = outputDir + File.separator
						+ diagram.getName() + ".c";
				generateSourceFile(templateConfig, templateRootMap, sourceFilename);
				System.out.println("Generated " + sourceFilename);

				// wb_t@C̐
				String headerFilename = outputDir + File.separator
						+ diagram.getName() + ".h";
				generateHeaderFile(templateConfig, templateRootMap, headerFilename);
				System.out.println("Generated " + headerFilename);
			}
		} finally {
			if (finder != null) {
				finder.close();
			}
		}
	}

	// ev[g̃RtBOs
	private static Configuration createTemplateConfig() throws IOException {

		final String projDir = CourseCodeGenerator.class.getProtectionDomain()
				.getCodeSource().getLocation().getPath();
		
		Configuration cfg = new Configuration();
		cfg.setDirectoryForTemplateLoading(new File(projDir
				+ File.separator + "../template"));
		cfg.setObjectWrapper(new DefaultObjectWrapper());
		return cfg;
	}

	// ev[ĝ߂̌𐶐
	private static Map<String, Object> createTemplateRootMap(
			IStateMachineDiagram diagram) {
		Map<String, Object> root = new HashMap<String, Object>();
		root.put("prefix", "EI");
		root.put("lastUpdated", new Date());
		root.put("diagram", diagram);
		return root;
	}

	// \[Xt@C𐶐
	private static void generateSourceFile(Configuration templateConfig,
			Map<String, Object> templateRootMap, String sourceFilename)
			throws IOException, TemplateException {
		
		Writer sourceWriter = new FileWriter(sourceFilename);
		Template sourceTemplate = templateConfig.getTemplate("course.ftl");
		sourceTemplate.process(templateRootMap, sourceWriter);
		sourceWriter.flush();
	}

	// wb_t@C𐶐
	private static void generateHeaderFile(Configuration templateConfig,
			Map<String, Object> templateRootMap, String headerFilename)
			throws IOException, TemplateException {
		
		Writer headerWriter = new FileWriter(headerFilename);
		Template headerTemplate = templateConfig.getTemplate("courseHeader.ftl");
		headerTemplate.process(templateRootMap, headerWriter);
		headerWriter.flush();
	}
}
